#include "xiicps.h"

XIicPs_Config XIicPs_ConfigTable[] __attribute__ ((section (".drvcfg_sec"))) = {

	{
		"cdns,i2c-r1p14", /* compatible */
		0xff020000, /* reg */
		0x5f5e0a4, /* xlnx,clock-freq */
		0x400e, /* interrupts */
		0xf9000000, /* interrupt-parent */
		0x62 /* clocks */
	},
	{
		"cdns,i2c-r1p14", /* compatible */
		0xff030000, /* reg */
		0x5f5e0a4, /* xlnx,clock-freq */
		0x400f, /* interrupts */
		0xf9000000, /* interrupt-parent */
		0x63 /* clocks */
	},
	 {
		 NULL
	}
};